﻿using System;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using System.IO;
using System.Data;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Net;

namespace  CenteringOnMovingVehicleWithTolerance
{
    public partial class TestForm : Form
    {
        private StreamReader mGPSData = new StreamReader(@"..\..\data\GPSinfo.txt");
        private Timer timer;
        private Timer centreTimer = new Timer();
        private double previousLong;
        private double previousLat;
        double latitude;
        double longitude;
        double angle;
        private RotationProjection rotateProjection = new RotationProjection();
        WorldMapKitWmsDesktopOverlay worldMapKitDesktopOverlay = new WorldMapKitWmsDesktopOverlay();
        LayerOverlay vehicleOverlay = new LayerOverlay();
        SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay();
        DragInteractiveOverlay dragInteractiveOverlay = new DragInteractiveOverlay();
        bool NewLine = true;
        int i = 1;
        LineShape line1 = new LineShape();

        public TestForm()
        {
            InitializeComponent();
            timer = new Timer();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            timer.Interval = 10; 
            timer.Tick += new EventHandler(timer_Tick);

            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.CurrentExtent = new RectangleShape(-97.7591, 30.3126, -97.7317, 30.2964);
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));

            worldMapKitDesktopOverlay.WebProxy = new System.Net.WebProxy("proxy.crc.ca");
            worldMapKitDesktopOverlay.DrawingExceptionMode = DrawingExceptionMode.DrawException;
            winformsMap1.Overlays.Add(worldMapKitDesktopOverlay);

            InMemoryFeatureLayer carLayer = new InMemoryFeatureLayer();
            carLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            carLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(@"..\..\data\sedan.png");
            carLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.RotationAngle = 45;
            carLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            carLayer.InternalFeatures.Add("Car", new Feature(new PointShape()));

            markerOverlay.MapControl = winformsMap1;
            markerOverlay.DragMode = MarkerDragMode.Drag;


            Vertex vb = new Vertex(-97.7591, 30.3126);
            Vertex vb1 = new Vertex(-97.7591, 30.3);
            line1.Vertices.Add(vb);
            line1.Vertices.Add(vb1);
            line1.Id = "line1";
            dragInteractiveOverlay.EditShapesLayer.InternalFeatures.Add(new Feature(line1));

            //Sets the PointStyle for the non dragged control points.
            dragInteractiveOverlay.ControlPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.PaleGoldenrod), new GeoPen(GeoColor.StandardColors.Black), 8);
            //Sets the PointStyle for the dragged control points.
            dragInteractiveOverlay.DraggedControlPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.Red), new GeoPen(GeoColor.StandardColors.Orange, 2), 10);

            dragInteractiveOverlay.CanAddVertex = false;
            dragInteractiveOverlay.CanDrag = true;
            dragInteractiveOverlay.CanRemoveVertex = false;
            dragInteractiveOverlay.CanResize = false;
            dragInteractiveOverlay.CanRotate = false;
            dragInteractiveOverlay.CalculateAllControlPoints();

            vehicleOverlay.Layers.Add("CarLayer", carLayer);

            winformsMap1.Overlays.Add("VehicleOverlay", vehicleOverlay);

            winformsMap1.Overlays.Add("MarkerOverlay", markerOverlay);

            winformsMap1.EditOverlay = dragInteractiveOverlay;
            dragInteractiveOverlay.CalculateAllControlPoints();

            winformsMap1.Refresh();

       }


        void timer_Tick(object sender, EventArgs e)
        {
            //Gets the GPS info from the textfile.
            DataTable carData = GetCarData();


            // Get the Row of Data we are working with.
            DataRow carDataRow = carData.Rows[0];

             latitude = Convert.ToDouble(carDataRow["LAT"]);
             longitude = Convert.ToDouble(carDataRow["LONG"]);

            if (previousLong == 0)
            {
                previousLong = longitude;
                previousLat = latitude;
            }

            double Xdiff = previousLong - longitude;
            double Ydiff = previousLat - latitude;

            //Gets the angle based on the current GPS position and the previous one to get the direction of the vehicle.
            angle = GetAngleFromTwoVertices(new Vertex(previousLong, previousLat), new Vertex(longitude, latitude));

            previousLong = longitude;
            previousLat = latitude;


            LayerOverlay vehicleOverlay = (LayerOverlay)winformsMap1.Overlays["VehicleOverlay"];
            InMemoryFeatureLayer carLayer = vehicleOverlay.Layers["CarLayer"] as InMemoryFeatureLayer;

            PointShape pointShape = carLayer.InternalFeatures[0].GetShape() as PointShape;
            carLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.RotationAngle = 90 - (float)angle;

            pointShape.X = longitude;
            pointShape.Y = latitude;
            pointShape.Id = "Car";

            carLayer.Open();
            carLayer.EditTools.BeginTransaction();
            carLayer.EditTools.Update(pointShape);
            carLayer.EditTools.CommitTransaction();
            carLayer.Close();
            


            if (Track.Checked == true)
            {
                winformsMap1.CenterAt(new PointShape(longitude, latitude));
            }
            winformsMap1.Refresh(winformsMap1.Overlays["VehicleOverlay"]);

        }

        private DataTable GetCarData()
        {
            DataTable datatable = new DataTable();
            datatable.Columns.Add("LAT");
            datatable.Columns.Add("LONG");

            string strLattitude = "";
            string strLongitude = "";

            // Read the next line from the text file with GPS data in it.
            string strCurrentText = mGPSData.ReadLine();

            if (strCurrentText == "")
            {
                mGPSData.BaseStream.Seek(0, SeekOrigin.Begin);
                strCurrentText = mGPSData.ReadLine();
            }

            while (strCurrentText != null)
            {
                // Every other line is a "/" and we want to skip those.
                if (strCurrentText.Trim() != "/")
                {
                    string[] strSplit = strCurrentText.Split(','); // (':');
                    strLongitude = strSplit[0];
                    strLattitude = strSplit[1];
                    break;
                }
                strCurrentText = mGPSData.ReadLine();
            }

            object[] objs = new object[2] { strLattitude, strLongitude };

            datatable.Rows.Add(objs);

            return datatable;
        }

        //We assume that the angle is based on a third point that is on top of b on the same x axis.
        private double GetAngleFromTwoVertices(Vertex b, Vertex c)
        {
            double alpha = 0;
            double tangentAlpha = (c.Y - b.Y) / (c.X - b.X);
            double Peta = Math.Atan(tangentAlpha);

            if (c.X > b.X)
            {
                alpha = 90 - (Peta * (180 / Math.PI));
            }
            else if (c.X < b.X)
            {
                alpha = 270 - (Peta * (180 / Math.PI));
            }
            else
            {
                if (c.Y > b.Y) alpha = 0;
                if (c.Y < b.Y) alpha = 180;
            }
            return alpha;
        }

      
        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;

            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);
            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
     
        private void winformsMap1_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            if (checkBox1.Checked == true)
            {
                Marker marker = new Marker(e.WorldLocation);
                marker.Image = Properties.Resources.downarrow;
                marker.Width = 20;
                marker.Height = 34;
                marker.YOffset = -17;
                markerOverlay.Markers.Add(marker);
                winformsMap1.Refresh(winformsMap1.Overlays["MarkerOverlay"]);
            }

            /*
            if (checkBox4.Checked == true && NewLine == true)
                {
                    LineShape line = new LineShape();
                    dragInteractiveOverlay.EditShapesLayer.InternalFeatures.Add("line" + i, new Feature(line));
                    i++;
                    NewLine = false;
                }
            
            if (checkBox4.Checked == true && NewLine == false)
            {
                Vertex v = new Vertex(e.WorldLocation);
                //LineShape line = dragInteractiveOverlay.EditShapesLayer.InternalFeatures[0].GetShape() as LineShape;
                line.Vertices.Add(v);

                /*line.Id = "line" + (i - 1);
                dragInteractiveOverlay.EditShapesLayer.FeatureSource.Open();
                dragInteractiveOverlay.EditShapesLayer.FeatureSource.BeginTransaction();
                dragInteractiveOverlay.EditShapesLayer.FeatureSource.UpdateFeature(line);
                dragInteractiveOverlay.EditShapesLayer.FeatureSource.CommitTransaction();
                dragInteractiveOverlay.EditShapesLayer.FeatureSource.Close();

                */

            if (checkBox4.Checked == true)
            {
                Vertex v = new Vertex(e.WorldLocation);
                //LineShape line1 = dragInteractiveOverlay.EditShapesLayer.InternalFeatures[0].GetShape() as LineShape;
                line1.Vertices.Add(v);
                //line1.Id = "Line1";

                /*
                dragInteractiveOverlay.EditShapesLayer.Open();
                dragInteractiveOverlay.EditShapesLayer.EditTools.BeginTransaction();
                dragInteractiveOverlay.EditShapesLayer.EditTools.Update(line1);
                dragInteractiveOverlay.EditShapesLayer.EditTools.CommitTransaction();
                dragInteractiveOverlay.EditShapesLayer.Close();
                */
                dragInteractiveOverlay.CalculateAllControlPoints();
                winformsMap1.Refresh();
            }


                winformsMap1.Refresh();


            
        }


        private void winformsMap1_Click(object sender, EventArgs e)
        {
           
        }

        private void checkBox1_CheckedChanged_1(object sender, EventArgs e)
        {
            if (checkBox1.Checked == true)
            {
                winformsMap1.MapClick += new EventHandler<MapClickWinformsMapEventArgs>(winformsMap1_MapClick);
            }

            if (checkBox1.Checked == false)
            {
                winformsMap1.MapClick -= new EventHandler<MapClickWinformsMapEventArgs>(winformsMap1_MapClick);
            }
        }

        private void checkBox2_CheckedChanged_1(object sender, EventArgs e)
        {
            if (checkBox2.Checked == true)
            {
                timer.Start();
            }

            if (checkBox2.Checked == false)
            {
                timer.Stop();
            }
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void Track_CheckedChanged(object sender, EventArgs e)
        {

        }


    }
}
